(*
Authors: 
		Gemma Ragozzine
		 Jon Paul
		 John Heizelman
		 Steven Weiner
*)

{ open Parser }

let digit = ['0'-'9']
let id = ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']*

rule token = parse 
[' ' '\t' 'r' '\n'] { token lexbuf }
| "/*" { comment lexbuf }
| '(' { LPAREN }
| '{' { LBRACE }
| '[' { LBRACK }
| ';' { SEMI }
| ':' { COLON }
| ')' { RPAREN }
| '}' { RBRACE }
| ']' { RBRACK }
| '%' { MOD }
| ',' { COMMA }
| '.' { PERIOD }
| '+' { PLUS }
| '*' { TIMES }
| "->" { LINK }
| "<->" { BILINK }
| '-' { MINUS }
| '/' { DIVIDE }
| "==" { EQ }
| '=' { ASSIGN }
| '.' {PERIOD}
| "!=" { NEQ }
| "<=" { LEQ }
| ">=" { GEQ }
| '<' { LT }
| '>' { GT }
| '!' { NOT }
| "&&" {AND}
| "||" {OR}
| "if" { IF }
| "else" { ELSE }
| "while" { WHILE }
| "for" { FOR }
| "return" { RETURN }
| "Graph" { GRAPH }
| "Node" { NODE }
| "Edge" { EDGE }
| "bool" { BOOL }
| "String" { STRING }
| "new" { NEW }
| "continue" { CONTINUE }
| "double" { DOUBLE }
| "int_at" { INTAT}
| "String_at" { STRINGAT }
| "char_at" { CHARAT }
| "bool_at" { BOOLAT}
| "int" { INT }
| "void" { VOID }
| "dest" { DEST }
| "edges" { EDGES }
| "static" { STATIC }
| "char" { CHAR }
| "in" { IN }
| "add" { ADD }
| ("true" | "false") as lxm	{ BOOLLIT(bool_of_string lxm) }
| '\"' ([^'\"']* as lxm) '\"'   { STRINGLIT(lxm) }
| ''' [ ^'''] as ch ''' { CHARLIT(ch) }
| ['0'-'9']+ as lxm { LITERAL(int_of_string lxm)}
| ['a'-'z' 'A' - 'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']*"[]" as lxm { ARRID(lxm)}
| ['a'-'z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { ID(lxm)}
| ['A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { TYPEID(lxm)}
| eof { EOF }
| _ as char { raise(Failure("illegal character " ^ Char.escaped char))}

and comment = parse
"*/" { token lexbuf }
| _ { comment lexbuf }
